// -*- Mode: c++ -*-
// $Id: Face.h,v 1.4 2000/04/28 22:14:20 ps Exp $
// $Source: /ufs/multires/projects//src/meshfilter/Face.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech)
 *
 * April 26, 2000
 * All Rights Reserved
 *
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 *
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef __FACE_H_
#define __FACE_H_

#include "Names.h"
#include "cvec.h"
#include "compat.h"
#include <vector>
#include <map>

class Vert;
class V4Vert;
struct VVPair;

class Face {

 public:
  Face(Vert * verts[], unsigned int size);
  ~Face();

  // compute the the central corner of the Face
  void Update( void );

  // For BIQ, the boundary is thrown away. the boundary shrinks.
  unsigned int CountPolygons( Scheme s ){
    return s == DS ? 1 : ( s == BIQ && !BoundaryOneRing() ? 1 : 0 );
  }

  // format outputs
  void WriteFace(Scheme, ostream *, std::map<const CVec*, unsigned long>*) const;
  void WriteFaceIV(Scheme, ostream *, std::map<const CVec*, unsigned long>*, const char *) const;

 private:
  // the central corner
  CVec * m_c;
  // the verts and their children
  std::vector<struct VVPair *> m_v;
  // whether this Face is on the boundary
  bool BoundaryOneRing( void ) const;
};

#endif /* __FACE_H_ */
