// -*- Mode: c++ -*-
// $Id: Mesh.h,v 1.4 2000/04/28 22:14:21 ps Exp $
// $Source: /ufs/multires/projects//src/meshfilter/Mesh.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech)
 *
 * April 26, 2000
 * All Rights Reserved
 *
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 *
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef __MESH_H_
#define __MESH_H_

class Vert;
class Face;

#include "Names.h"
#include "compat.h"
#include <vector>
#include <stdio.h>

class Mesh{
 public:
  Mesh( ::Scheme sch ) : scheme(sch), m_vc(), m_fc() 
    { assert(sch == DS || sch == BIQ); }
  ~Mesh( void );

  void Subdiv( void );
  ::Scheme Scheme() const { return scheme; }

  // output the V4 mesh in smf format
  ostream& WritePrimalPolygon( ostream& os ) const;
  // output the V4 mesh in iv format
  ostream& WritePrimalPolygonIV( ostream& os ) const;

  // readin an iv file
  friend bool readIVFile( FILE* fp, Mesh& m );
  // formated i/o streams
  friend ostream& operator<<( ostream& os, Mesh& m )
    { return m.WritePrimalPolygonIV( os ); }
  // input is smf format
  friend istream& operator>>( istream& is, Mesh& m);

 private:
  ::Scheme scheme;
  // all Verts
  std::vector<Vert *> m_vc;
  // all Faces
  std::vector<Face *> m_fc;

};

#endif /* __MESH_H_ */
