// -*- Mode: c++ -*-
// $Id: V4Vert.h,v 1.4 2000/04/28 22:14:21 ps Exp $
// $Source: /ufs/multires/projects//src/meshfilter/V4Vert.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech)
 *
 * April 26, 2000
 * All Rights Reserved
 *
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 *
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef __V4VERT_H_
#define __V4VERT_H_

#include "Names.h"
#include "cvec.h"
#include "compat.h"
#include <assert.h>

class Vert;

// valence-4 vertex
class V4Vert{
 public:
  V4Vert(Vert * v);
  ~V4Vert();

  Vert * Parent() { return m_p; }
  const CVec& Loc( void ) const { return m_l; }
  // compute the position of this V4Vert. it should be the average of
  // its four corners
  void Update( void ) { 
    assert( m_d[NW] && m_d[SW] && m_d[SE] && m_d[NE] );
    m_l=((*m_d[NW]) + (*m_d[SW]) + (*m_d[SE]) + (*m_d[NE]))/4.0; 
  }
  void Link(const Direction d, V4Vert * v){
    // never re-link a neighbor
    assert(!m_n[d]); m_n[d] = v; 
  }
  void UnLink(const Direction d) { 
    // never unlink a null
    assert(m_n[d]); m_n[d] = 0; 
  }
  void SetCorner(const QTreeName n, CVec* c) { 
    // never "reset" a corner. So either a corner is initially assigned, or
    // it's set to NULL.
    assert(!m_d[n] || !c); m_d[n] = c; 
  }
  CVec * Corner(const QTreeName n) const { return m_d[n]; }
  bool AllCorners( void ) const { return m_d[NW] && m_d[SW] && m_d[SE] && m_d[NE]; }

  V4Vert * Neighbor(const Direction d) const { return m_n[d]; }
  bool AllNeighbors( void ) const { return m_n[N] && m_n[W] && m_n[S] && m_n[E]; }

 private:
  // the parent
  Vert * m_p;
  // the four neighbors
  V4Vert * m_n[4];
  // dual slots
  CVec * m_d[4];
  // the geometric location
  CVec m_l;
};

#endif /* __V4VERT_H_ */
