// -*- Mode: c++ -*-
// $Id: Vert.h,v 1.4 2000/04/28 22:14:22 ps Exp $
// $Source: /ufs/multires/projects//src/meshfilter/Vert.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech)
 *
 * April 26, 2000
 * All Rights Reserved
 *
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 *
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef __VERT_H_
#define __VERT_H_

class Face;
class V4Vert;
struct V4OPair;

#include "Names.h"
#include "cvec.h"
#include "compat.h"
#include <vector>
#include <map>

class Vert{

 public:
  Vert(const CVec& l) : m_l(l), m_v() {}
  ~Vert();

  void AddChild(V4Vert * c);

  // build up the connectivity of the children and neighbor Verts
  void BuildTopology( void );
  // compute the children's corners
  void Update( void );
  // compute children
  void Subdiv( void );
  CVec& Loc( void ) { return m_l; }
  // if this Vert is on the boundary, this method will return false
  bool CompleteOneRing( void ) const;

  // count the number of vertices and polygons for output
  unsigned int CountVerts( Scheme sheme ) const;
  unsigned int CountPolygons( Scheme scheme ) const;

  // format output
  unsigned long WriteVerts(Scheme, ostream *, std::map<const CVec*, unsigned long>*, unsigned long) const;
  unsigned long WriteVertsIV(Scheme, ostream *, std::map<const CVec*, unsigned long>*, unsigned long, const char *) const;
  void WriteFace(Scheme, ostream *, std::map<const CVec*, unsigned long>*) const;
  void WriteFaceIV(Scheme, ostream *, std::map<const CVec*, unsigned long>*, const char *) const;

 private:
  // the actual geometric location
  CVec m_l;
  // children and the orientation of incident faces
  std::vector<struct V4OPair *> m_v;
};

#endif /* __VERT_H_ */
