// $Id: compat.h,v 1.4 2000/04/28 22:14:22 ps Exp $
// $Source: /ufs/multires/projects//src/meshfilter/compat.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech)
 *
 * April 26, 2000
 * All Rights Reserved
 *
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 *
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef __COMPAT_H__
#define __COMPAT_H__

#include <assert.h>

// turn out the visual studio warnings (identifier was truncated) 

#ifdef _WIN32
#pragma warning(disable:4786)
#endif

// currently, SGI's STL does not have the new-style iostream 
#if defined(_WIN32) || defined(__GNUG__) || defined(_STANDARD_C_PLUS_PLUS)
#include <iostream>
#include <fstream>
#include <strstream>
#else 
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#endif


// 2.8.1 gcc does not support namespaces
// in other cases need this to avoid std:: prefixes
#if ( defined(_WIN32) || defined(__sgi) ) && !defined(__GNUG__)
namespace std {}
using namespace std;
#endif

// standard C++ does not seem to have bidirectional_iterator
// defined; however, both gcc and SGI CC have it
#ifdef _WIN32
template <class T,class dist> 
class bidirectional_iterator: public iterator<bidirectional_iterator_tag,T,dist>
{};
#endif

// includes min/max declaration

#if (defined(__sgi) || defined(__GNU__))
#include <algobase.h>
#endif

// dealing with Microsoft min/max mess: 
//assume that under Windows the code is compiled with NOMINMAX defined
// which disables #define's for min/max.
// however, Microsoft  violates the C++ standard even with 
// NOMINMAX on, and defines templates _cpp_min and _cpp_max 
// instead of templates min/max
// define the correct templates here

#ifdef _WIN32

namespace std {
	template<class _Ty> inline
        const _Ty& max(const _Ty& _X, const _Ty& _Y)
        {return (_X < _Y ? _Y : _X); }

	template<class _Ty> inline
        const _Ty& min(const _Ty& _X, const _Ty& _Y)
        {return (_Y < _X ? _Y : _X); }
};

#endif

// disable VC6.0 warning "identifier was truncated to 255 characters "
// this is routine whe compiling templates
// unfortunately, it does not seem to work in all cases
// and the messages are still visible when compiling STL

#ifdef _WIN32
#pragma warning(disable:4786)
#define M_PI		3.14159265358979323846
#endif

#endif /* __COMPAT_H__ */


