// -*- Mode: c++ -*-
// $Id: cvec.cpp,v 1.4 2000/04/28 22:14:22 ps Exp $
// $Source: /ufs/multires/projects//src/meshfilter/cvec.cpp,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech)
 *
 * April 26, 2000
 * All Rights Reserved
 *
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 *
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

// do not edit anything above this line

#include <math.h>
#include <assert.h>

#include "cvec.h"

void 
CVec::print(ostream& os) const
{
  os << m_v[0] << " " << m_v[1] << " " << m_v[2];
}

ostream&
operator<<( ostream& os, const CVec& c )
{
  c.print( os );	
  return os;
}

istream&
operator>>( istream& is, CVec& f )
{
  return is >> f.m_v[0] >> f.m_v[1] >> f.m_v[2];
}


bool
CVec::read( istream& is )  {
  return ( is >> m_v[0]) && (is >> m_v[1]) && (is  >> m_v[2]);
}

CVec
CVec::dir( void ) const
{
  real a = l2();
  if( zero( a ) ) return *this;
  else return *this / a;
}

CVec
CVec::perp( void ) const
{
  CVec nrm = *this;
  if( !nrm.normalize() ) return CVec( 1, 0, 0 );

  CVec pnrm;
  // some component must be < 1/(sqrt(3))
  if( fabs( nrm.m_v[X] ) < real(0.6) ) pnrm = cross( CVec( 1, 0, 0 ) );
  else if( fabs( nrm.m_v[Y] ) < real(0.6) ) pnrm = cross( CVec ( 0, 1, 0 ) );
  else if( fabs( nrm.m_v[Z] ) < real(0.6) ) pnrm = cross( CVec ( 0, 0, 1 ) );
  else die();
  
  return pnrm.dir();
}

int
CVec::collinear( const CVec& a, const CVec& b ) const
{
  return zero( ( a - *this ).cross( a - b ).dot() );
}

