// -*- Mode: c++ -*-
// $Id: cvec.h,v 1.4 2000/04/28 22:14:22 ps Exp $
// $Source: /ufs/multires/projects//src/meshfilter/cvec.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech)
 *
 * April 26, 2000
 * All Rights Reserved
 *
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 *
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef	__CVEC_H__
#define	__CVEC_H__

// Do not change anything above this line

#include <math.h>
#include <algorithm>
#include "real.h"
#include "util.h"
#include "compat.h"

//class ostream;
//class istream;
class CVec;
//below two forward declarations of friends are included for Visual C++ bug 
istream& operator>>( istream&, CVec& );
ostream& operator<<( ostream&, const CVec& );

// column type 3 vectors
class CVec{
  public:
  static const CVec Zero;
  enum{
    X = 0,
    Y = 1,
    Z = 2
    };
  CVec( void ) {}
  CVec( const real f ) { m_v[X] = m_v[Y] = m_v[Z] = f; }
  CVec( const CVec& c )
    { m_v[X] = c.m_v[X], m_v[Y] = c.m_v[Y], m_v[Z] = c.m_v[Z]; }
  CVec( const real a, const real b, const real c )
    { m_v[X] = a, m_v[Y] = b, m_v[Z] = c; }
  CVec( const float* a )
    { m_v[X] = real( a[X] ), m_v[Y] = real( a[Y] ), m_v[Z] = real( a[Z] ); }
  CVec( const double* a )
    { m_v[X] = real( a[X] ), m_v[Y] = real( a[Y] ), m_v[Z] = real( a[Z] ); }
  operator real*( void )
    { return &m_v[X]; }
  operator const real*( void ) const
    { return &m_v[X]; }
  // ~CVec( void ) {}
  
  friend istream& operator>>( istream&, CVec& );
  friend ostream& operator<<( ostream&, const CVec& );
  void print( ostream& os ) const;
  bool read ( istream& is );

  bool operator==(const CVec& c)const
    { return m_v[X]==c.m_v[X] && m_v[Y]==c.m_v[Y] && m_v[Z]==c.m_v[Z];}
  bool operator<(const CVec& c)const 
    { return unsigned(this)<unsigned(&c); }
  
  CVec& operator=( const CVec& c )
    { m_v[X] = c.m_v[X], m_v[Y] = c.m_v[Y], m_v[Z] = c.m_v[Z]; return *this; }
  CVec operator+( const CVec& c ) const
    { return CVec( m_v[X] + c.m_v[X], m_v[Y] + c.m_v[Y], m_v[Z] + c.m_v[Z] ); }
  CVec operator-( const CVec& c ) const
    { return CVec( m_v[X] - c.m_v[X], m_v[Y] - c.m_v[Y], m_v[Z] - c.m_v[Z] ); }
  CVec operator*( const real s ) const
    { return CVec( s * m_v[X], s * m_v[Y], s * m_v[Z] ); }
  friend CVec operator*( const real s, const CVec& c )
    { return CVec( s * c.m_v[X], s * c.m_v[Y], s * c.m_v[Z] ); }
  CVec operator/( const real s ) const
    { return CVec( m_v[X] / s, m_v[Y] / s, m_v[Z] / s ); }

  CVec& operator+=( const CVec& c )
    { m_v[X] += c.m_v[X], m_v[Y] += c.m_v[Y], m_v[Z] += c.m_v[Z]; return *this; }
  CVec& operator-=( const CVec& c )
    { m_v[X] -= c.m_v[X], m_v[Y] -= c.m_v[Y], m_v[Z] -= c.m_v[Z]; return *this; }
  CVec& operator*=( const real s )
    { m_v[X] *= s, m_v[Y] *= s, m_v[Z] *= s; return *this; }
  CVec& operator/=( const real s )
    { m_v[X] /= s, m_v[Y] /= s, m_v[Z] /= s; return *this; }
  CVec operator-( void ) const
    { return CVec( -m_v[X], -m_v[Y], -m_v[Z] ); }
  
  //stupid but useful
  CVec operator*(const CVec& c)const{
    return CVec(m_v[X]*c.m_v[X], m_v[Y]*c.m_v[Y], m_v[Z]*c.m_v[Z]);
  }
  CVec operator/(const CVec& c)const{
    return CVec(m_v[X]/c.m_v[X], m_v[Y]/c.m_v[Y], m_v[Z]/c.m_v[Z]);
  }
  
  real& x( void ) { return m_v[X]; }
  real x( void ) const { return m_v[X]; }
  real& y( void ) { return m_v[Y]; }
  real y( void ) const { return m_v[Y]; }
  real& z( void ) { return m_v[Z]; }
  real z( void ) const { return m_v[Z]; }
  real& operator() (const int i)      { return m_v[i]; }
  real operator() (const int i) const { return m_v[i]; }
  
  CVec lerp( const CVec& v1, const real t ) const
    { return CVec( m_v[X] + t * ( v1.m_v[X] - m_v[X] ),
		   m_v[Y] + t * ( v1.m_v[Y] - m_v[Y] ),
		   m_v[Z] + t * ( v1.m_v[Z] - m_v[Z] ) ); }

  CVec min( const CVec& o ) const
  { real a = std::min( m_v[X], o.m_v[X] );
	real b = std::min( m_v[Y], o.m_v[Y] );
	real c = std::min( m_v[Z], o.m_v[Z] );
    return CVec( a, b, c ); }
  CVec max( const CVec& o ) const
  {		real a = std::max( m_v[X], o.m_v[X] );
      real b = std::max( m_v[Y], o.m_v[Y] );
      real c = std::max( m_v[Z], o.m_v[Z] );
      return CVec( a, b, c ); }
  void bbox( CVec& min, CVec& max ){
    if( m_v[X] < min.m_v[X] ) min.m_v[X] = m_v[X];
    else if( m_v[X] > max.m_v[X] ) max.m_v[X] = m_v[X];
    if( m_v[Y] < min.m_v[Y] ) min.m_v[Y] = m_v[Y];
    else if( m_v[Y] > max.m_v[Y] ) max.m_v[Y] = m_v[Y];
    if( m_v[Z] < min.m_v[Z] ) min.m_v[Z] = m_v[Z];
    else if( m_v[Z] > max.m_v[Z] ) max.m_v[Z] = m_v[Z];
  }

  real dot( const CVec& c ) const
    { return m_v[X] * c.m_v[X] + m_v[Y] * c.m_v[Y] + m_v[Z] * c.m_v[Z]; }
  real dot( void ) const
    { return m_v[X] * m_v[X] + m_v[Y] * m_v[Y] + m_v[Z] * m_v[Z]; }
  CVec cross( const CVec& c ) const
    { return CVec( m_v[Y] * c.m_v[Z] - m_v[Z] * c.m_v[Y],
		  m_v[Z] * c.m_v[X] - m_v[X] * c.m_v[Z],
		  m_v[X] * c.m_v[Y] - m_v[Y] * c.m_v[X] ); }
  real l1( void ) const
    { real a = abs( m_v[X] ); a += abs( m_v[Y] ); return a + abs( m_v[Z] ); }
  real linfty( void ) const
    { real a = abs( m_v[X] ); a = std::max( a, abs( m_v[Y] ) );
      return std::max( a, abs( m_v[Z] ) ); }
  real l2( void ) const { return sqrtf( dot() ); }
  
  CVec dir( void ) const;

  int normalize( void )
    { real mag = l2(); return zero( mag ) ? 0 : ( *this *= 1 / mag, 1 ); }
  real dist( const CVec& c ) const { return ( *this - c ).l2(); }
  int collinear( const CVec& a, const CVec& b ) const;
  CVec perp( void ) const;
  
 protected:
 private:
  real	m_v[3];
};

typedef CVec* CVecp;

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#endif	/* __CVEC_H__ */
