// -*- mode: c++ -*-
// $Id: CMesh.h,v 1.11 2000/04/28 22:14:46 ps Exp $
// $Source: /ufs/multires/projects//src/CourseQuad/CMesh.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech) 
 *
 * April 26, 2000
 * All Rights Reserved
 * 
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 * 
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef __CMESH_H
#define __CMESH_H

#include <stdio.h>
#include <vector>

#include "Names.h"
#include "util.h"
#include "real.h"
#include "compat.h"

class Cp;
class CFace;
class CMesh;

// below two forward declarations of friends are included for Visual C++ bug 
istream& operator>>( istream&, CMesh& );
bool readIVFile( FILE* fp, CMesh& m );

class CMesh{
public:

  // Constructor / Destructor
  CMesh( Scheme sch ) : m_depth(0), m_cpc(), m_fc(), m_s(sch) {}
  ~CMesh( void );

  // formatted i/o
  friend ostream& operator<<( ostream& os, CMesh& m ){
    return m.m_adaptive ? m.WriteDualTriangleIV( os ) : m.WritePrimalPolygonIV( os );
  }
  friend istream& operator>>( istream&, CMesh& );

  // input the mesh from an iv file
  friend bool readIVFile( FILE* fp, CMesh& m );

  // subdivision part (negative error threshold enforces subdivision always)
  void Subdiv( const unsigned int todepth, const real eps = -1 );

  // output of the mesh
  // these are example functions for a very simple file format that can
  // serve as examples for other file formats
  ostream& WritePrimalPolygon( ostream &os );
  ostream& WriteDualTriangle( ostream &os );
  // these functions depend on the qvlib
  ostream& WritePrimalPolygonIV( ostream& os );
  ostream& WriteDualTriangleIV( ostream& os );

private:
  unsigned int m_depth;
  // all control points
  std::vector<Cp*> m_cpc;
  // all faces
  std::vector<CFace *> m_fc;

  // the scheme associated with this mesh
  Scheme m_s;
  // is subdivision adaptive?
  bool m_adaptive;

  // it's not clear what the meaning of these would be...
  CMesh( const CMesh& ) { die(); }
  CMesh& operator=( const CMesh& ) { die(); return *this; }
};

#endif /* __CMESH_H */
