// -*- mode: c++ -*-
// $Id: Cp.cpp,v 1.11 2000/04/28 22:14:46 ps Exp $
// $Source: /ufs/multires/projects//src/CourseQuad/Cp.cpp,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech) 
 *
 * April 26, 2000
 * All Rights Reserved
 * 
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 * 
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#include "Cp.h"
#include "QTree.h"
#include "Perms.h"

Cp::Cp( const CVec& p, Scheme sch )
  : m_s(sch)
{
  m_n[N] = m_n[W] = m_n[S] = m_n[E] = 0;
  m_o[N] = m_o[W] = m_o[S] = m_o[E] = true;

  CVec *corners[4] = {0,0,0,0};
  m_root = new QTree(this, NULL, corners, p); assert( m_root );
}

Cp::~Cp( void )
{
  delete m_root;
  m_root = 0;

  if( m_n[N] ) m_n[N]->Unlink( this );
  if( m_n[W] ) m_n[W]->Unlink( this );
  if( m_n[S] ) m_n[S]->Unlink( this );
  if( m_n[E] ) m_n[E]->Unlink( this );
  assert( !m_n[N] && !m_n[W] && !m_n[S] && !m_n[E] );
}


/**
 * given the direction "d", return the permutation of the neighboring
 * Cp to "perm", retrieve the name table "names" and return the reflected 
 * name table to the long int
 */

bool
Cp::Permute( unsigned long& names, const int height, const Direction d, Perms * perm )
{
  // get the proper Cp pointer and permutation
  Cp *cp = Neighbor(d);
  // nothing to permute, nobody there
  if( !cp ) return false;

  // first time, figure out the actual permutation
  const Perms p = Perms( d, cp->Neighbor(this), m_o[d] );

  // whoever wants to know, let them have the vertex permutation map
  if( perm ) *perm = p;

  unsigned int mask = 0x3;
  // go through in groups of 2 bits and use them as offsets into the
  // permutation table
  for( int i = 0, shift = 0; i < height; i++, shift += 2, mask <<= 2 ){
    // manipulate 2 bits each time. map the names in the old QTree to the
    // permuted names in the neighboring QTree
    names = ( names & ~mask ) | ( p[QTreeName(( names & mask ) >> shift)] << shift );
  }

  return true;
}

#include "compat.h"

ostream& operator<<( ostream& os, const Cp& ){ return os; }
istream& operator>>( istream& is, Cp& ){ return is; }
