// -*- Mode: c++ -*-
// $Id: Cp.h,v 1.10 2000/04/28 22:14:46 ps Exp $
// $Source: /ufs/multires/projects//src/CourseQuad/Cp.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech) 
 *
 * April 26, 2000
 * All Rights Reserved
 * 
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 * 
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef	__CP_H__
#define	__CP_H__

// Do not change anything above this line

// include files
#include "real.h"
#include "Names.h"
#include "util.h"
#include "cvec.h"
#include "compat.h"


class QTree;
class Perms;

// class definition
class Cp{
 public:

  Cp( const CVec& p, Scheme sch );
  ~Cp( void );
  // destructor cleanup
  void Unlink( const Cp *cp ){
    const Direction d = Neighbor(cp);
    for( int i = 0; i < 4; ++i ) if( m_n[d]->m_n[i] == this ) m_n[d]->m_n[i] = 0;
    m_n[d] = 0;
  }

  // link neighbor with null permutation
  void Link( Cp *cp, const Direction d ){ assert(cp); assert(cp!=this); m_n[d] = cp; }

  // formatted i/o
  friend ostream& operator<<( ostream&, const Cp& );
  friend istream& operator>>( istream&, Cp& );

  // accessors
  QTree* Root( void ) const { return m_root; }
  QTree*& Root( void ) { return m_root; }

  // get Cp neighbor by symbolic direction
  Cp* Neighbor( const Direction d ) const { assert( 0 <= d && d <= 3 ); return m_n[d]; }
  Cp*& Neighbor( const Direction d ) { assert( 0 <= d && d <= 3 ); return m_n[d]; }

  // reverse lookup
  Direction Neighbor( const Cp *cp ) const{
    return m_n[N]==cp ? N: ( m_n[W]==cp ? W: ( m_n[S]==cp ? S: ( assert(m_n[E]==cp), E ) ) );
  }

  // flip the orientation of the neighbor
  void Flip(const Direction d){ m_o[d] = false; }
  
  bool Permute( unsigned long& names, const int height, const Direction d, Perms * perm = 0 );
  
  Scheme scheme( void ) const { return m_s; }

 private:
  QTree *m_root;
  // neighbors in four directions
  Cp *m_n[4];
  // orientation parity (only matters for non-orientable manifolds) and is
  // associated with the edges connecting to the neighbors; true means that
  // they are both oriented the same way, false means they are not
  bool m_o[4];

  // indicates the scheme of subdivision
  Scheme m_s;

  Cp( const Cp& ) { die(); }
  Cp& operator=( const Cp& ) { die(); return *this; }
};

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#endif	/* __CP_H__ */
