// -*- Mode: c++ -*-
// $Id: OneRing.h,v 1.5 2000/04/28 22:14:46 ps Exp $
// $Source: /ufs/multires/projects//src/CourseQuad/OneRing.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech) 
 *
 * April 26, 2000
 * All Rights Reserved
 * 
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 * 
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef	__ONERING_H__
#define	__ONERING_H__

// Do not change anything above this line

// include files

#include "Names.h"
#include "OneRing.h" 
#include "real.h"
#include "cvec.h"
#include "compat.h"

class QTree;

// class definition
class OneRing{
 public:
  OneRing( QTree *t, const Scheme sch );
  // ~OneRing( void ) {}

  // does the child actually exist?
  bool c( const QTreeName qtn ) const{ return m_i[qtn] >= 0; }

  // will there be any children?
  operator bool( void ) const {
    return m_s == BIQ ?
      ( m_i[NW] >= 0 && m_i[SW] >=0 && m_i[SE] >= 0 && m_i[NE] >= 0 ) :
      ( m_i[NW] >= 0 || m_i[SW] >=0 || m_i[SE] >= 0 || m_i[NE] >= 0 );
  }

  // the subdivision function
  CVec Subdiv( const QTreeName qtn ) const {
    return m_s == DS ? DooSabin(qtn) :
      ( m_s == ME ? MidEdge(qtn) : ( bool(*this) ? Biquartic(qtn) : DooSabin(qtn) ) );
  }
  bool GetCorners( const QTreeName d, CVec *corner[4] );
  
  const QTree * Neighbor( const Direction d ) const { return m_t[d]; }
  int Size( const QTreeName qtn ) const { return m_i[qtn]; }
  QTree* V( const QTreeName qtn, const int i ){
    assert( qtn == NW || qtn == SW || qtn == SE || qtn == NE );
    assert( i >= 0 && i < m_i[qtn] );
    return m_q[qtn][i];
  }
  // what is the approximation error of the children of t?
  real Eps( QTree * ) const;

 protected:
 private:
  // center of the 1-ring
  QTree *m_c;
  // neighbors in the 4 major directions
  QTree *m_t[4];
  // how many are in the respective chains?
  // if value is -1, no face in this direction
  int m_i[4];
  // the quadrant chains
  QTree *m_q[4][__MAXFACESIZE];
  
  Scheme m_s;

  CVec MidEdge( const QTreeName ) const;
  CVec DooSabin( const QTreeName ) const;
  CVec Biquartic( const QTreeName ) const;

  // coefficient tables for Doo-Sabin, MidEdge and Bi-Quartic
  void BuildTables( void ) const;

  static bool m_init;
  static real m_ds[__MAXFACESIZE][__MAXFACESIZE];
  static real m_me[__MAXFACESIZE][__MAXFACESIZE];
  static real m_bq[4][__MAXFACESIZE][__MAXFACESIZE];
  // fixed coefficients for bq
  static real m_fbq[4][8];
};

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#endif	/* __ONERING_H__ */
