// -*- Mode: c++ -*-
// $Id: Perms.h,v 1.12 2000/04/28 22:14:47 ps Exp $
// $Source: /ufs/multires/projects//src/CourseQuad/Perms.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech) 
 *
 * April 26, 2000
 * All Rights Reserved
 * 
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 * 
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef __PERMS_H_
#define __PERMS_H_

#include "Names.h"
#include <assert.h>
#include "compat.h"

class Perms{
 public:

  // the default constructor, a NON-permutation
  Perms( void ) : m_p(-1) { }
  // return the right Perms object of the neighbor
  // d1 - the Direction the neighbor to "self"
  // d2 - the Direction "self" to the neighbor in the neighbor's view point
  // b  - same orientation?
  Perms( const Direction d1, const Direction d2, const bool b )
    : m_p(m_Ptbl[b][d1][d2])
    { assert( d1 >= N && d1 <= E && d2 >= N && d2 <= E ); }
  Perms( const Perms& p ) : m_p(p.m_p) {}
  ~Perms( void ) {}

  // to retrieve the permed direction or qtreename
  // BEAWARE: now if passing "int" value to [] operator, you must cast it to
  // either Direction or QTreeName
  Direction operator [] ( Direction d ) const{
    assert( m_p != -1 ); return m_dp[m_p][d];
  }
  QTreeName operator [] ( QTreeName qtn ) const{
    assert( m_p != -1 ); return m_qtnp[m_p][qtn];
  }

  Perms& operator=( const Perms& p ) { m_p = p.m_p; return *this; }

  // whether a permutation represented by this Perms has the same orientation
  // or the reflection
  // return true indicates reflected orientation
  // this operator closely associated with m_dp and m_qtnp
  bool Reflect ( void ) const{
    assert( m_p != -1 ); return m_p >= 4 && m_p <= 7;
  }

  // indicates whether this Perms is a real permuatation
  operator bool ( void ) const { return m_p != -1; }

  // why is this static?
  static Perms Ident( void ) { return Perms(0); }

 private:

  Perms( const int i ) : m_p(i) {}

  // the index of permutation defined by the two mapping arrays
  int m_p;

  // the mapping arrays
  static int m_Ptbl[2][4][4];
  static Direction m_dp[8][4];
  static QTreeName m_qtnp[8][4];
};
#endif /* __PERMS_H_ */
