// -*- Mode: c++ -*-
// $Id: QTree.h,v 1.10 2000/04/28 22:14:47 ps Exp $
// $Source: /ufs/multires/projects//src/CourseQuad/QTree.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech) 
 *
 * April 26, 2000
 * All Rights Reserved
 * 
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 * 
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef	__QTREE_H__
#define	__QTREE_H__

//class istream;
//class ostream;
class Perms;

#include "cvec.h"
#include "Names.h"
#include "real.h"
#include "compat.h"

// this is the class that manages neighborhood relations at the top level
// of the mesh
class Cp;

class QTree{
public:
  QTree( void ) {}
  QTree( Cp*                cp,
	 QTree*             parent,
	 CVec*              corners[4],
	 const CVec&        location,
	 const QTreeName    name = NW, // for the root this has no meaning...
	 const unsigned int depth = 0 );
  ~QTree( void );

  friend istream& operator>>( istream&, QTree& );
  friend ostream& operator<<( ostream&, const QTree& );
  
  bool Leaf( void ) const { return !m_c[NE] && !m_c[NW] && !m_c[SW] && !m_c[SE]; }
  bool CompleteSetOfChildren( void ) const { return m_c[NE] && m_c[NW] && m_c[SW] && m_c[SE]; }
  bool Root( void ) const { return m_p == 0; }
  
  // create 4 children (negative number on the error implies forced
  // subdivision no matter what)
  bool Subdiv( const real eps = -1 );
  // force expansion of the entire tree to depth 
  void Expand( const unsigned int depth, const real eps = -1 );

  // get a neighbor accross an edge
  QTree* Neighbor( Direction, Perms * perm = 0 ) const;

  // how deep is a tree  actually
  int QTreeDepth( void ) const;

  // the control point in the mesh which owns this tree
  const Cp* CP( void ) const   { return m_cp; }
  Cp*       CP( void )         { return m_cp; }	 
  // name
  QTreeName       n( void ) const { return QTreeName(m_n); }
  // depth
  unsigned int    Depth( void ) const { return m_depth; }

  // parent
  QTree*& Parent( void )       { return m_p; }
  QTree*  Parent( void ) const { return m_p; }

  // four children
  QTree*& ne( void )       { return m_c[NE]; }
  QTree*  ne( void ) const { return m_c[NE]; }
  QTree*& nw( void )       { return m_c[NW]; }
  QTree*  nw( void ) const { return m_c[NW]; }
  QTree*& sw( void )       { return m_c[SW]; }
  QTree*  sw( void ) const { return m_c[SW]; }
  QTree*& se( void )       { return m_c[SE]; }
  QTree*  se( void ) const { return m_c[SE]; }
  QTree*&  c( const QTreeName qtn ) { return m_c[qtn]; }
  QTree*   c( const QTreeName qtn ) const { return m_c[qtn]; }

  // the four corners
  void SetCorner( const QTreeName qtn, CVec * c ) 
    // is supposed to be called on Root's only
    { assert(Root()); m_corner[qtn] = c; }
  CVec* Corner( const QTreeName qtn ) const { return m_corner[qtn]; }
  QTreeName Corner( const CVec* c ) const {
    return m_corner[NW] == c ? NW : ( m_corner[SW] == c ? SW :
				      ( m_corner[SE] == c ? SE :
					( assert( m_corner[NE] == c ), NE ) ) );
  }

  const CVec& loc( void ) const { return m_l; }

  // service functions
  template <class F> F ForEachOddVertex( F f )
    {
      if( !Leaf() ){
	Perms np;
	const QTree *n;
	
	// center
	if( m_c[NW] && m_c[SW] && m_c[SE] && m_c[NE] ) f( m_c[NW]->m_corner[SE] );
	
	// western edge
	if( m_c[NW] && m_c[SW] ){
	  n = Neighbor( W, &np ); assert( n->Depth() == Depth() );
	  if( !( n->m_c[np[SE]] && n->m_c[np[NE]] ) || ( n < this ) ){
	    f( m_c[NW]->m_corner[SW] );
	  }
	}
	// southern edge
	if( m_c[SW] && m_c[SE] ){
	  n = Neighbor( S, &np ); assert( n->Depth() == Depth() );
	  if( !( n->m_c[np[NW]] && n->m_c[np[NE]] ) || ( n < this ) ){
	    f( m_c[SW]->m_corner[SE] );
	  }
	}
	// eastern edge
	if( m_c[SE] && m_c[NE] ){
	  n = Neighbor( E, &np ); assert( n->Depth() == Depth() );
	  if( !( n->m_c[np[SW]] && n->m_c[np[NW]] ) || ( n < this ) ){
	    f( m_c[SE]->m_corner[NE] );
	  }
	}
	// nothern edge
	if( m_c[NE] && m_c[NW] ){
	  n = Neighbor( N, &np ); assert( n->Depth() == Depth() );
	  if( !( n->m_c[np[SW]] && n->m_c[np[SE]] ) || ( n < this ) ){
	    f( m_c[NE]->m_corner[NW] );
	  }
	}
	if( m_c[NW] ) f = m_c[NW]->ForEachOddVertex( f );
	if( m_c[SW] ) f = m_c[SW]->ForEachOddVertex( f );
	if( m_c[SE] ) f = m_c[SE]->ForEachOddVertex( f );
	if( m_c[NE] ) f = m_c[NE]->ForEachOddVertex( f );
      }
      
      return f;
    }
  
  template <class F> F
  ForEachQTree( F f)
    {
      f( this );
      if( m_c[NW] ) f = m_c[NW]->ForEachQTree( f );
      if( m_c[SW] ) f = m_c[SW]->ForEachQTree( f );
      if( m_c[SE] ) f = m_c[SE]->ForEachQTree( f );
      if( m_c[NE] ) f = m_c[NE]->ForEachQTree( f );
      
      return f;
    }
  
protected:
private:
  // pointer to control point in the mesh (forest) we belong to
  Cp*    m_cp;
  // parent pointer
  QTree*           m_p;
  // child pointers
  QTree*           m_c[4];
  // the actual geometric location in space
  CVec             m_l;
  // the limit points in dual mesh
  CVec*            m_corner[4];
  // for adaptive subdivision we measure error
  real  m_eps;
  // depth
  unsigned int     m_depth:__MAXDEPTHLENGTH;
  // name of myself 
	// ought to be QTreeName but in Visual C++ the unsigned enums and bitfields
	// do not work well together
  unsigned int        m_n:2;
  
  // the qtree name permutation array helper storage
  // static Direction m_nodperm[4];
  // support maps for neighbor finding
  static QTreeName m_reflect[4/*Direction*/][4/*QTreeName*/];
  static bool m_opposite[4/*Direction*/][4/*QTreeName*/];
  static QTreeName m_crossover[4/*Direction*/][4/*QTreeName*/];

  // we just don't know what the semantics of this should be...
  QTree( const QTree& ) { die(); }
  QTree operator=( const QTree& ) { die(); return QTree(); }
};


#endif	/* __QTREE_H__ */
