// -*- Mode: c++ -*-
// $Id: util.h,v 1.6 2000/04/28 22:14:47 ps Exp $
// $Source: /ufs/multires/projects//src/CourseQuad/util.h,v $

/**
 * Copyright 1995-2000, Peter Schr\"oder, California Institute of
 * Technology (Caltech) 
 *
 * April 26, 2000
 * All Rights Reserved
 * 
 * The use of this software is permitted for non-commercial, educational,
 * and research use only. The software and/or related materials are
 * provided "as-is" without warranty of any kind including any warranties
 * of performance or merchantability or fitness for a particular use or
 * purpose (as set forth in UCC 23212-2313) or for any purpose
 * whatsoever, for the licensed product, however used. In no event shall
 * Caltech be liable for any damages and/or costs, including but not
 * limited to incidental or consequential damages of any kind, including
 * economic damage or injury to property and lost profits, regardless of
 * whether Caltech shall be advised, have reason to know, or in fact
 * shall know of the possibility. User bears all risk relating to quality
 * and performance of the software and/or related materials.
 * 
 * Any use other than non-commercial, educational, or research, or any
 * redistribution in original or modified form requires prior written
 * authorization from the copyright holder.
 */

#ifndef	__UTIL_H__
#define	__UTIL_H__

#include <assert.h>
#include "compat.h"

#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif

#ifdef _WIN32
#define fatan2(x,y) atan2(x,y)
#endif

#ifdef _WIN32
#include <float.h>
#define isnanf(x) (_isnan(x))
#endif

// Inline function definitions go here
template <class _type>
inline int
argmin( const _type a, const _type b, const _type c )
{
  return a <= b ? ( a <= c ? 0 : 2 ) : ( b <= c ? 1 : 2 );
}

template <class _type>
inline int
sign( const _type f )
{
  return f < 0 ? -1 : ( f > 0 ? 1 : 0 );
}

template <class _type>
inline _type
abs( const _type f )
{
  return f < 0 ? -f : f;
}

template <class _type>
inline _type
sqr( const _type s )
{
  return s * s;
}

template <class _type>
inline _type
cube( const _type s )
{
  return s * s * s;
}

inline bool
even( const int i )
{
  return !( i % 2 );
}

inline bool
odd( const int i )
{
  return (i % 2) != 0;
}

inline bool
zero( const float x, const float eps )
{
  return ( -eps < x ) && ( x < eps );
}

inline bool
zero( const float x )
{
  static float __negeps_f = -1e-6f;
  static float __poseps_f =  1e-6f;

  return ( __negeps_f < x ) && ( x < __poseps_f );
}

inline bool
zero( const double x, const double eps )
{
  return ( -eps < x ) && ( x < eps );
}

inline bool
zero( const double x )
{
  static double __negeps_d = -1e-16f;
  static double __poseps_d =  1e-16f;

  return ( __negeps_d < x ) && ( x < __poseps_d );
}

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#include <time.h>
// for timing
static clock_t __timerstart, __timerend;

inline void InitTimer( void ) { clock(); }
inline void StartTimer( void ) { __timerstart = clock(); }
inline void EndTimer( void ) { __timerend = clock(); }

inline ostream&
PrintDeltaTimer( ostream& os )
{
  os << ( 1000.0f* ( float )( __timerend - __timerstart ) / (CLOCKS_PER_SEC )) <<
    " msec";
  return os;
}

#include <assert.h>
// a special version with diagnostics
// modeled after assert.h
#ifdef __cplusplus
extern "C" {
#endif
#ifdef NDEBUG

#undef assertzero
#define assertzero(EX) ((void)0)
#define die() ((void)0)
#else

#ifdef __ANSI_CPP__
#define assertzero(EX)  ((zero(EX))?((void)0):\
			 (cerr.precision(12),\
			  cerr << "assertion on zero( " <<\
			  # EX << " failed by " <<\
			  (EX)/m_eps_f << " eps\n",\
			 __assert( # EX , __FILE__, __LINE__)))
#else
#define assertzero(EX)  ((zero(EX))?((void)0):\
			 (cerr.precision(12),\
			  cerr << "assertion on zero( " <<\
			  "EX" << " failed by " <<\
			  (EX)/m_eps_f << " eps\n",\
			 __assert( "EX", __FILE__, __LINE__)))
#endif

// objectcenter doesn't like __assert
#ifdef _WIN32
#define die()            (_assert( "disallowed", __FILE__, __LINE__ ))
#else
#define die()            (__assert( "disallowed", __FILE__, __LINE__ ))
#endif
//#define die()            (assert(0)) 

#endif /* NDEBUG */

#ifdef __cplusplus
}
#endif

#endif	/* __UTIL_H__ */
